
# q-expansion of newform 3024.2.a.g, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3024
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-1], [-1], [0], [-6], [7], [8], [-2], [-4], [2], [9], [-11], [7], [-11], [-10], [9], [-8], [0], [-12], [-14], [-1], [-9], [-14], [4], [-6], [6], [6], [-5], [-6], [1], [4], [2], [-12], [-8], [5], [0], [-7], [21], [14], [-26], [20], [-22], [-9], [12], [16], [-20], [-6], [-8], [8], [-22], [-2], [-14], [-1], [-6], [-6], [3], [-2], [-23], [6], [16], [9], [26], [-15], [-4], [-6], [-29], [-1], [18], [20], [27], [-30], [-14], [23], [-15], [-15], [4], [-14], [12], [-24], [31], [-2], [-24], [-2], [36], [16], [-26], [34], [9], [1], [-12], [11], [8], [-6], [-31], [-31], [-9], [3], [-2], [1], [23], [4], [44], [-18], [7], [0], [44], [3], [-16], [10], [-10], [-26], [6], [12], [37], [-32], [-22], [24], [38], [16], [-22], [-2], [-42], [6], [34], [-38], [-29], [45], [52], [4], [20], [-20], [40], [19], [33], [-32], [25], [2], [-30], [-18], [-6], [24], [13], [-18], [-16], [45], [-20], [21], [-46], [22], [17], [22], [-19], [-39], [37], [32], [-43], [49], [-36], [15], [-28], [34], [0], [11], [-48], [9], [51], [-32], [9], [61], [8], [48], [-6], [-5], [0], [-13], [52], [0], [-28], [-2], [-61], [-5], [-14], [-17], [-48], [-27], [38], [10], [6], [-45], [-34], [24], [13], [-51], [-40], [-57], [10], [-37], [-14], [-1], [-28], [-12], [-34], [46], [57], [-15], [-56], [0], [2], [-58], [-27], [18], [35], [-45], [-5], [36], [15], [-10], [-18], [64], [27], [14], [-46], [39], [-37], [65], [9], [-38], [38], [-42], [-50], [-2], [-37], [-56], [26], [0], [44], [12], [-46], [-3], [40], [-68], [46], [10], [-30], [-43], [2], [-41], [9], [10], [53], [38], [62], [-54], [-11], [0], [-25], [10], [54], [18], [-8], [18], [12], [-56], [66], [-32], [-20], [-50], [33], [-18], [70], [-4], [14], [15], [-54], [-56], [34], [-53], [-61], [15], [3], [-17], [21], [-36], [-2], [36], [-51], [-32], [-66], [33], [30], [-25], [32], [11], [-1], [-11], [42], [-1], [-9], [-44], [-34], [12], [-48], [54], [45], [2], [-12], [22], [-82], [-70], [64], [-70], [46], [53], [4], [34], [-67], [62], [40], [48], [-74], [-91], [69], [32], [-10], [-70], [47], [56], [72], [12], [-66], [-24], [80], [-49], [13], [-83], [-62], [61], [-16], [32], [-69], [12], [39], [40], [62], [72], [-32], [35], [23], [62], [58], [1], [42], [24], [-21], [51], [60], [1], [-84], [24], [-85], [-15], [42], [60], [48], [-31], [-45], [-21], [67], [-32], [-74], [24], [82], [26], [78], [68], [18], [38], [50], [88], [-47], [-88], [-64], [54], [-50], [-24], [-20], [74], [-59], [20], [-1], [58], [90], [-42], [-100], [38], [26], [-65], [20], [-66], [24], [-4], [69], [-15], [26], [-20], [17], [-38], [48], [0], [36], [18], [-2], [65], [75], [-54], [74], [-52], [-33], [32], [-66], [-64], [-18], [9], [78], [-75], [-4], [-12], [-2], [32]]
